/*
Package pkg provides the high-level, real main function of the pkg tool `Run`
as well as a single place where all constants are defined.

See the user documentation for more information about the pkg tool. As
mentioned there, you can change some of it's configuration by modifying the
constants in `config.go`.

Alternatively, you can also embed/wrap `Run` itself. However, this is not meant
to be a library. Generally useful functions to interact programmatically with a
repository of SlackBuilds can be found in the `lib/sboRepo` package on which
this package is built on.
*/
package pkg
